
 { to construct right hand side of the equations of the motion }

 UNIT  UnForces ;

 INTERFACE

 Uses
   UnConTyp ; { constants TVect3 TMatr33 CharSelF }

 Procedure ClcRightHandCoor ( utc      : Extended ;
                              rc , vc  : TVect3 ;
                              Var
                                force  : TVect3 ) ;

 Procedure ToSelectForce ( chf      : Char ;
                           utc      : Extended ;
                           rc , vc  : TVect3 ;
                           Var
                             force  : TVect3 ) ;

 IMPLEMENTATION

 Uses
   UnForRom , { FromFixToTerraM FromTerraToFIxM }
   UnForFun , { TranspMatr procedure }
   UnForPnm , { ClcPrecMatr procedure }
   UnForceM , { TheMainForce }
   UnForceH , { ForceDueToGeoHarm }
   UnForceP , { ForceDueToPlanets }
   UnForAtm ; { ForceDueToAtm }

 { to calculate the Forces }

 Procedure ClcRightHandCoor ( utc      : Extended ;
                              rc , vc  : TVect3 ;
                              Var
                                force  : TVect3 ) ;
 Var
   teph   : Extended ; { ephemeris time TDB }
   ct,tc  : TMatr33 ; { matrixes to link celestial and terrestrial }
   cf,fc  : TMatr33 ; { matrixes to link fixed and mean equator system }
 Begin

   { matrixes to link terrestrial and celestial system }
   FromFixToTerraM(utc,0.0,teph,ct); { UnForRom for TDB and matrix }
   TranspMatr(ct,tc);    { UnForFun for matrix from TRS to CRS }

   { matrix to link fixed and mean equator system }
   ClcPrecMatr(MJD2000,teph,cf); { UnForPnm  precession matrix }
   TranspMatr(cf,fc);    { UnForFun  from mean to fixed equator }

   { the Earth as a material point with a mass M }
   TheMainForce(rc,force); { from UnForceZ }

   If  CharSelF = 'P'  Then  Exit ; { only main force }

   { to add zonal , tesseral and sectorial harmonics }
   ForceDueToGeoHarm(rc,ct,tc,force); { from UnForceH }

   If  CharSelF = 'H'  Then  Exit ; { only geopotential }

   { to add the beautiful Moon }
   ForceDueToMoon(teph,rc,fc,force); { from UnForceP }

   { to add the yellow Sun }
   ForceDueToSun(teph,rc,fc,force);  { from UnForceP }

   { to add Solar radiation pressure }
   ForceDueToSolPres(teph,rc,fc,force); { from UnForceP }

   { to add forces due to atmospheric drag }
   ForceDueToAtm(teph,rc,vc,force); { from UnForAtm with velosity }

 End;

 { to calculate the selected Forces }

 Procedure ToSelectForce ( chf      : Char ;
                           utc      : Extended ;
                           rc , vc  : TVect3 ;
                           Var
                             force  : TVect3 ) ;
 Var
   ic     : Byte ;
   teph   : Extended ; { ephemeris time TDB }
   ct,tc  : TMatr33 ; { matrixes to link celestial and terrestrial }
   cf,fc  : TMatr33 ; { matrixes to link fixed and mean equator system }
 Begin

   For ic:=1 To 3 Do  force[ic]:=0.0; { initial nullo }

   { matrixes to link terrestrial and celestial system }
   FromFixToTerraM(utc,0.0,teph,ct); { UnForRom for TDB and matrix }
   TranspMatr(ct,tc);    { UnForFun for matrix from TRS to CRS }

   { matrix to link fixed and mean equator system }
   ClcPrecMatr(MJD2000,teph,cf); { UnForPnm  precession matrix }
   TranspMatr(cf,fc);    { UnForFun  from mean to fixed equator }

   { the Earth as a material point with a mass M }
   If  chf = 'P'  then  TheMainForce(rc,force); { from UnForceZ }

   { to add zonal , tesseral and sectorial harmonics }
   If  chf = 'H'  then  ForceDueToGeoHarm(rc,ct,tc,force); { from UnForceH }

   { to add the beautiful Moon }
   If  chf = 'M'  then  ForceDueToMoon(teph,rc,fc,force); { from UnForceP }

   { to add the yellow Sun }
   If  chf = 'S'  then  ForceDueToSun(teph,rc,fc,force);  { from UnForceP }

   { to add Solar radiation pressure }
   If  chf = 'R'  then  ForceDueToSolPres(teph,rc,fc,force); { UnForceP }

   { to add forces due to atmospheric drag }
   If  chf = 'A'  then  ForceDueToAtm(teph,rc,vc,force);  { UnForAtm }

 End;

 END.
