
unit UnForVec ; { for condition vector }

interface

uses
  UnConTyp ; { for type TVect3 }

procedure CondToCart ( dist , velt , thet  : Extended ;
                       incl , node , argu  : Extended ;
                       var    vecr , vecv  : TVect3  ) ;

procedure ConzToCart ( dist , velt , thet  : Extended ;
                       incz , nodz , argz  : Extended ;
                       var    vecr , vecv  : TVect3  ) ;

procedure CartToCond ( vecr , vecv  : TVect3 ;
                       var  dist , velt , thet  : Extended ;
                       var  incl , node , argu  : Extended ) ;

procedure CartToConz ( vecr , vecv  : TVect3 ;
                       var  dist , velt , thet  : Extended ;
                       var  incz , nodz , argz  : Extended ) ;

implementation
uses
  UnForFun ; { for function DATan2 }

{ to change from equator
  condition vector to DesCart position and velosity
  InPut
   dist  : range in km
   velt  : linear velosity in km/s
   thet  : angle between linear velosity and transversal to orbit radian
   incl  : angle of inclination radian
   node  : the ascending node in radian
   argu  : argument of latitude in radian
  OutPut
   vecr  : array[1..3] position vector in km
   vecv  : array[1..3] velosity vector in km/s }

procedure CondToCart ( dist , velt , thet  : Extended ;
                       incl , node , argu  : Extended ;
                       var    vecr , vecv  : TVect3  ) ;
var
  si,ci,sn,cn,su,cu,vr  : Extended ;
  xr,yr,zr,xv,yv,zv,vn  : Extended ;
begin
  si:=Sin(incl); { for inclination }
  ci:=Cos(incl);
  sn:=Sin(node); { for ascending node }
  cn:=Cos(node);
  su:=Sin(argu); { for argument of latitude }
  cu:=Cos(argu);
  vr:=velt*Sin(thet);
  vn:=velt*Cos(thet);
  xr:=+cu*cn-su*sn*ci;
  yr:=+cu*sn+su*cn*ci;
  zr:=+su*si;
  xv:=-su*cn-cu*sn*ci;
  yv:=-su*sn+cu*cn*ci;
  zv:=+cu*si;
  vecr[1]:=dist*xr;
  vecr[2]:=dist*yr;
  vecr[3]:=dist*zr;
  vecv[1]:=vr*xr+vn*xv;
  vecv[2]:=vr*yr+vn*yv;
  vecv[3]:=vr*zr+vn*zv;
end;

{ to change from meridian
  condition vector to DesCart position and velosity
  InPut
   dist   : range in km
   velt   : linear velosity in km/s
   thet   : angle between linear velosity and transversal to orbit radian
   incl*  : angle of inclination radian    meridian
   node*  : the ascending node in radian   meridian
   argu*  : argument of latitude in radian meridian
  OutPut
   vecr  : array[1..3] position vector in km
   vecv  : array[1..3] velosity vector in km/s }

procedure ConzToCart ( dist , velt , thet  : Extended ;
                       incz , nodz , argz  : Extended ;
                       var    vecr , vecv  : TVect3  ) ;
var
  si,ci,sn,cn,su,cu,vr  : Extended ;
  xr,yr,zr,xv,yv,zv,vn  : Extended ;
begin
  si:=Sin(incz); { for inclination  meridian }
  ci:=Cos(incz);
  sn:=Sin(nodz); { for ascending node  meridian }
  cn:=Cos(nodz);
  su:=Sin(argz); { for argument of latitude  meridian }
  cu:=Cos(argz);
  vr:=velt*Sin(thet);
  vn:=velt*Cos(thet);
  xr:=+cu*sn+su*cn*ci;
  yr:=+su*si;
  zr:=+cu*cn-su*sn*ci;
  xv:=-su*sn+cu*cn*ci;
  yv:=+cu*si;
  zv:=-su*cn-cu*sn*ci;
  vecr[1]:=dist*xr;
  vecr[2]:=dist*yr;
  vecr[3]:=dist*zr;
  vecv[1]:=vr*xr+vn*xv;
  vecv[2]:=vr*yr+vn*yv;
  vecv[3]:=vr*zr+vn*zv;
end;

{ to change from
  DesCart position and velosity to equator condition vector
  InPut
   vecr  : array[1..3] position vector in km
   vecv  : array[1..3] velosity vector in km/s
  OutPut
   dist  : range in km
   velt  : linear velosity in km/s
   thet  : angle between linear velosity and transversal to orbit radian
   incl  : angle of inclination radian
   node  : the ascending node in radian
   argu  : argument of latitude in radian }

procedure CartToCond ( vecr , vecv  : TVect3 ;
                       var  dist , velt , thet  : Extended ;
                       var  incl , node , argu  : Extended ) ;
var
  fm,fp,st,ct,vr,vn  : Extended ;
  c1,c2,c3,l1,l2,l3  : Extended ;
  ac,al,ae,ap,si,ci  : Extended ;
  sn,cn,su,cu,sv,cv  : Extended ;
  sp,cp,xr,yr,zr     : Extended ;
begin
  fm:=398600.4415; { geocentric gravitational constant km^3/s^2 }
  dist:=Sqrt(Sqr(vecr[1])+Sqr(vecr[2])+Sqr(vecr[3])); { range in km }
  xr:=vecr[1]/dist; { x component for unit vector of position }
  yr:=vecr[2]/dist; { y component for unit vector of position }
  zr:=vecr[3]/dist; { z component for unit vector of position }
  velt:=Sqrt(Sqr(vecv[1])+Sqr(vecv[2])+Sqr(vecv[3])); { velosity in km/s }
  c1:=vecr[2]*vecv[3]-vecr[3]*vecv[2];  { three momentum integrals }
  c2:=vecr[3]*vecv[1]-vecr[1]*vecv[3];
  c3:=vecr[1]*vecv[2]-vecr[2]*vecv[1];
  ac:=Sqrt(c1*c1+c2*c2+c3*c3);          { modul }
  l1:=-fm*xr+vecv[2]*c3-vecv[3]*c2; { three integrals Laplas }
  l2:=-fm*yr+vecv[3]*c1-vecv[1]*c3;
  l3:=-fm*zr+vecv[1]*c2-vecv[2]*c1;
  al:=Sqrt(l1*l1+l2*l2+l3*l3);          { modul }
  ae:=al/fm;      { eccentricity of orbit }
  ap:=Sqr(ac)/fm; { parametr of the orbit }
  fp:=Sqrt(fm/ap);
  ci:=c3/ac;               { cosinus of the inclination }
  si:=Sqrt(Abs(1-Sqr(ci))); { sinus of the inclination }
  if  si < 1.0e-11
    then
      begin
        si:=1.0e-11;
        incl:=0;            { singularity inclination = 0 }
      End
    Else
      incl:=DATan2(si,ci); { inclination in radian }
  sn:=+c1/(ac*si);     { for ascending node }
  cn:=-c2/(ac*si);
  node:=DATan2(sn,cn); { the ascending node in radian }
  sp:=l3/(al*si);      { for argument of perigei }
  cp:=(l1/al)*cn+(l2/al)*sn;
  su:=zr/si;           { sinus argument of latitude }
  cu:=xr*cn+yr*sn;     { cosinis the same }
  argu:=DATan2(su,cu); { argument of latitude in radian }
  sv:=su*cp-cu*sp;     { sinus of true anomaly }
  cv:=cu*cp+su*sp;     { cosinus of true anomaly }
  vr:=fp*ae*sv;        { radial velosity }
  vn:=fp*(1+ae*cv);    { transversal velosity }
  st:=vr/velt;         { sinus theta }
  ct:=vn/velt;         { cosinus theta }
  thet:=DATan2(st,ct);  { angle theta in radian }
end;

{ to change from
  DesCart position and velosity to meridian condition vector
  InPut
   vecr  : array[1..3] position vector in km
   vecv  : array[1..3] velosity vector in km/s
  OutPut
   dist  : range in km
   velt  : linear velosity in km/s
   thet  : angle between linear velosity and transversal to orbit radian
   incz  : angle of inclination radian    meridian
   nodz  : the ascending node in radian   meridian
   argz  : argument of latitude in radian meridian }

procedure CartToConz ( vecr , vecv  : TVect3 ;
                       var  dist , velt , thet  : Extended ;
                       var  incz , nodz , argz  : Extended ) ;
var
  r,v  : TVect3 ;
begin
  r[1]:=vecr[3];
  r[2]:=vecr[1];
  r[3]:=vecr[2];
  v[1]:=vecv[3];
  v[2]:=vecv[1];
  v[3]:=vecv[2];
  CartToCond(r,v,dist,velt,thet,incz,nodz,argz);
end;

end.
