unit UnJob123 ; { for numeric integration with Solar pressure }
interface
procedure ForJob123 ;
implementation
uses
  UnConTyp , { for type TVect3 }
  UnForDat , { for FromDatetoMJD function }
  UnForKep , { FromKeplerToCart procedure }
  UnForTim , { for TransUTCtoTDT TransTDTtoTDB procedures }
  UnForRom , { for FromTrueToFixEqu procedure }
  UnForFun , { for MultMatrVec procedure }
  UnHarmCS , { fro coefficients }
  UnForSop , { for global variable SatReflCoef }
  UnEveRim , { for PosVelEverGrator procedure }
  UnForVec , { ConToCart procedure }
  UnJobWrt ; { for write mean equator position }
procedure ForJob123 ; { with Solar radiation pressure }
var
  nds,nms    : Integer ; { start date }
  nys        : Integer ; { day month year }
  khs,kms    : Integer ; { hour minute }
  secs       : Extended ; { second }
  ndf,nmf    : Integer ; { finish date }
  nyf        : Integer ; { day month year }
  khf,kmf    : Integer ; { hour minute }
  secf       : Extended ; { second }
  tmjs,tmjf  : Extended ; { moment in modified julian day }
var
  pose,ange  : TVect3 ;   { Kepler elements }
  vecr,vecv  : TVect3 ;   { rectangular position velocity }
  recr,recv  : TVect3 ;   { the same }
  dist,velt  : Extended ; { components of the condition vector }
  thet,incl  : Extended ;
  node,argu  : Extended ;
  mprt       : Word ;
  step,ster  : Extended ; { step in second for integration }
  tbeg,tend  : Extended ;
  rbeg,vbeg  : TVect3 ;
  rend,vend  : TVect3 ;
  posk,angk  : TVect3 ;   { Kepler elements }
var
  delut      : Extended ; { TT-UTC }
  tdt,tdb    : Extended ; { scales of time }
  truecel    : TMatr33 ;  { type from UnConTyp }
  celtrue    : TMatr33 ;  { from fixed to true equator }
begin
  WriteLn;
  WriteLn('  12.3 To prognos by numeric integration');
  WriteLn('         to take into account Solar radiation pressure');
  nds:=31;
  nms:=3;
  nys:=2005;
  khs:=5;
  kms:=45;
  secs:=0.0;
  tmjs:=FromDateToMJD(nds,nms,nys,khs,kms,secs); { UnForDat }
  ndf:=9;
  nmf:=4;
  nyf:=2005;
  khf:=15;
  kmf:=51;
  secf:=0.0;
  tmjf:=FromDateToMJD(ndf,nmf,nyf,khf,kmf,secf); { UnForDat in UTC scale }
  pose[1]:=FromPeriodToA(86000.0); { period in second semi-major axis km }
  pose[2]:=0.03;    { eccentricity }
  pose[3]:=1.2;     { inclination in degree }
  ange[1]:=172.1;    { the ascending node in degree }
  ange[2]:=215.0;   { argument of perigei in degree }
  ange[3]:=150.0;   { mean anomaly in degree }
  FromKeplerToCart(pose,ange,vecr,vecv); { from UnForKep }
  CartToConz(vecr,vecv,dist,velt,thet,incl,node,argu); { UnForVec }
  ConzToCart(dist,velt,thet,incl,node,argu,recr,recv); { UnForVec }
  { initial condition
    from true equatorial system to celestial cyctem }
  TransUTCtoTDT(tmjs,tdt,delut);  { UnForTim to TT scale }
  TransTDTtoTDB(tdt,tdb);         { UnForTim to Teph scale }
  FromTrueToFixEquM(tdb,truecel);  { UnForRom for matrix }
  MultMatrVec(truecel,recr,vecr); { UnForRom for position }
  MultMatrVec(truecel,recv,vecv); { UnForRom for velocity }
  HarmCSValue ;    { from UnHarmCS for coefficients }
  SatReflCoef:=ToGetReflValue(1.1,2.0,400.0);  { UnForSop }
  SatBallCoef:=0;  { UnForSop }
  step:=900;       { step in second for geosynchronous }
  EverInit ;       { from UnEveRiM just all }
  EverCoefNullo ;  { from UnEveRiM just all }
  tend:=tmjs; { to start moment in UTC scale }
  rend:=vecr;
  vend:=vecv;
  WriteLn('  InPut');
  WriteLn('   ',1.0e6*SatReflCoef:19:12,
          '  efficient coefficient of reflection in mm/s^2');
  WriteLn('  start moment');
  WriteLn('  Moscow decret winter time');
  WriteLn('  ',nds:10,'      day');
  WriteLn('  ',nms:10,'      month');
  WriteLn('  ',nys:10,'      year');
  WriteLn('  ',khs:10,'      hour');
  WriteLn('  ',kms:10,'      minute');
  WriteLn('  ',secs:14:3,'  second');
  WriteMeanEquPos(tmjs,vecr,vecv);    { from UnJobWrt to mean equator }
  WriteLn('   condition vector (meridian place)');
  WriteLn('   ',dist:19:3,' ':19,'  range in km');
  WriteLn('   ',velt:19:6,' ':19,'  velocity in km/s');
  WriteLn('   ',thet:19:10,RadGra*thet:19:6,'  angle theta  radian degree');
  WriteLn('   ',incl:19:10,RadGra*incl:19:6,'  inclination* radian degree');
  WriteLn('   ',node:19:10,RadGra*node:19:6,'  ascend.node* radian degree');
  WriteLn('   ',argu:19:10,RadGra*argu:19:6,'  arglatitude* radian degree');
  WriteLn('  finish moment');
  WriteLn('  Moscow decret winter time');
  WriteLn('  ',ndf:10,'      day');
  WriteLn('  ',nmf:10,'      month');
  WriteLn('  ',nyf:10,'      year');
  WriteLn('  ',khf:10,'      hour');
  WriteLn('  ',kmf:10,'      minute');
  WriteLn('  ',secf:14:3,'  second');
  WriteLn('  OutPut');
  WriteLn('  parameters in listing:');
  WriteLn('  moment in DAY from the start moment   D');
  WriteLn('  semi-major axis in km                 a');
  WriteLn('  eccentricity                          e');
  WriteLn('  inclination in degree                 i');
  WriteLn('  the ascending node in degree         OM');
  WriteLn('  the argument of perigei in degree    om');
  WriteLn('  the mean anomaly in degree            M');
  WriteLn('  D','a':14,'e':14,'i':11,'OM':13,'om':12,'M':12);
  mprt:=0;
  FromCartToKepler(rend,vend,posk,angk);
  WriteLn(Round(tend-tmjs):3,
          posk[1]:14:6,posk[2]:14:10,posk[3]:11:6,
          angk[1]:13:6,angk[2]:12:6,angk[3]:12:6);
  while  ( tend < tmjf ) do
    begin
      PosVelEverGrator(step,tbeg,rbeg,vbeg,tend,rend,vend);
      mprt:=mprt+1;
      if  mprt = 96
        then
          begin
            FromCartToKepler(rend,vend,posk,angk);
            WriteLn(Round(tend-tmjs):3,
                    posk[1]:14:6,posk[2]:14:10,posk[3]:11:6,
                    angk[1]:13:6,angk[2]:12:6,angk[3]:12:6);
            mprt:=0;
          end;
      if  ( ( tbeg <= tmjf ) and ( tmjf <= tend ) )
        then  { information for our moment }
          begin
            ster:=86400*(tmjf-tbeg); { interval in second }
            ClcVectFromPolinom(ster,rbeg,vbeg,recr,recv); { UnEveRim }
          end;
    end;
  TransMJDtoDate(tmjf,ndf,nmf,nyf,khf,kmf,secf); { finish moment UnForDat }
  { initial condition
    from celestial system to true equatorial system }
  TransUTCtoTDT(tmjf,tdt,delut);  { UnForTim to TT scale }
  TransTDTtoTDB(tdt,tdb);         { UnForTim to Teph scale }
  FromFixToTrueEquM(tdb,celtrue);  { UnForRom for matrix }
  MultMatrVec(celtrue,recr,vecr); { UnForRom for position }
  MultMatrVec(celtrue,recv,vecv); { UnForRom for velocity }
  CartToConz(vecr,vecv,dist,velt,thet,incl,node,argu); { UnForVec }
  WriteLn('   Our result');
  WriteMeanEquPos(tmjf,recr,recv);    { from UnJobWrt to mean equator }
  WriteLn('   ',tmjf:19:10,'  finish moment in modified julian day');
  WriteLn('   ',vecr[1]:19:3,'  x  coordinate in km');
  WriteLn('   ',vecr[2]:19:3,'  y  coordinate in km');
  WriteLn('   ',vecr[3]:19:3,'  z  coordinate in km');
  WriteLn('   ',vecv[1]:19:6,'  vx velocity in km/s');
  WriteLn('   ',vecv[2]:19:6,'  vy velocity in km/s');
  WriteLn('   ',vecv[3]:19:6,'  vz velocity in km/s');
  WriteLn;
  WriteLn('  finish moment');
  WriteLn('  Moscow decret winter time');
  WriteLn('  ',ndf:10,'      day');
  WriteLn('  ',nmf:10,'      month');
  WriteLn('  ',nyf:10,'      year');
  WriteLn('  ',khf:10,'      hour');
  WriteLn('  ',kmf:10,'      minute');
  WriteLn('  ',secf:14:3,'  second');
  WriteLn('   condition vector (meridian place)');
  WriteLn('   ',dist:19:3,' ':19,'  range in km');
  WriteLn('   ',velt:19:6,' ':19,'  velocity in km/s');
  WriteLn('   ',thet:19:10,RadGra*thet:19:6,'  angle theta  radian degree');
  WriteLn('   ',incl:19:10,RadGra*incl:19:6,'  inclination* radian degree');
  WriteLn('   ',node:19:10,RadGra*node:19:6,'  ascend.node* radian degree');
  WriteLn('   ',argu:19:10,RadGra*argu:19:6,'  arglatitude* radian degree');
end;

end.
